@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2017-2018 Codethink Limited
 #
@@ -19,4 +18,3 @@
 #        Tristan Van Berkom <tristan.vanberkom@codethink.co.uk>
 
 from .artifactcache import ArtifactCache, ArtifactCacheSpec
-from .artifactcache import artifact_cache_specs_from_config_node, configured_remote_artifact_cache_specs
