@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2016-2018 Codethink Limited
 #
@@ -20,7 +19,6 @@
 
 import os
 import sys
-import shutil
 import resource
 import traceback
 import datetime
@@ -32,19 +30,17 @@ import click
 from click import UsageError
 
 # Import buildstream public symbols
-from .. import Scope, Consistency
+from .. import Scope
 
 # Import various buildstream internals
 from .._context import Context
 from .._project import Project
-from .._exceptions import BstError, PipelineError, LoadError, LoadErrorReason, AppError
+from .._exceptions import BstError, StreamError, LoadError, LoadErrorReason, AppError
 from .._message import Message, MessageType, unconditional_messages
-from .._pipeline import Pipeline, PipelineSelection
-from .._scheduler import Scheduler
-from .._profile import Topics, profile_start, profile_end
+from .._stream import Stream
 from .._versions import BST_FORMAT_VERSION
-from .. import __version__ as build_stream_version
 from .. import _yaml
+from .._scheduler import ElementJob
 
 # Import frontend assets
 from . import Profile, LogLine, Status
@@ -69,9 +65,8 @@ class App():
         # Public members
         #
         self.context = None        # The Context object
+        self.stream = None         # The Stream object
         self.project = None        # The toplevel Project object
-        self.scheduler = None      # The Scheduler
-        self.pipeline = None       # The Pipeline
         self.logger = None         # The LogLine object
         self.interactive = None    # Whether we are running in interactive mode
         self.colors = None         # Whether to use colors in logging
@@ -80,10 +75,12 @@ class App():
         # Private members
         #
         self._session_start = datetime.datetime.now()
+        self._session_name = None
         self._main_options = main_options  # Main CLI options, before any command
         self._status = None                # The Status object
         self._fail_messages = {}           # Failure messages by unique plugin id
         self._interactive_failures = None  # Whether to handle failures interactively
+        self._started = False              # Whether a session has started
 
         # UI Colors Profiles
         self._content_profile = Profile(fg='yellow')
@@ -127,25 +124,57 @@ class App():
             # Set soft limit to hard limit
             resource.setrlimit(resource.RLIMIT_NOFILE, (limits[1], limits[1]))
 
-    # partially_initialized()
+    # create()
     #
-    # Early stage initialization context manager which only initializes the
-    # Context, Project and the logger.
+    # Should be used instead of the regular constructor.
     #
-    # partial initialization is useful for some contexts where we dont
-    # want to load the pipeline, such as executing workspace commands.
+    # This will select a platform specific App implementation
     #
     # Args:
-    #    fetch_subprojects (bool): Whether we should fetch subprojects as a part of the
-    #                              loading process, if they are not yet locally cached
+    #    The same args as the App() constructor
+    #
+    @classmethod
+    def create(cls, *args, **kwargs):
+        if sys.platform.startswith('linux'):
+            # Use an App with linux specific features
+            from .linuxapp import LinuxApp
+            return LinuxApp(*args, **kwargs)
+        else:
+            # The base App() class is default
+            return App(*args, **kwargs)
+
+    # initialized()
+    #
+    # Context manager to initialize the application and optionally run a session
+    # within the context manager.
+    #
+    # This context manager will take care of catching errors from within the
+    # context and report them consistently, so the CLI need not take care of
+    # reporting the errors and exiting with a consistent error status.
+    #
+    # Args:
+    #    session_name (str): The name of the session, or None for no session
+    #
+    # Note that the except_ argument may have a subtly different meaning depending
+    # on the activity performed on the Pipeline. In normal circumstances the except_
+    # argument excludes elements from the `elements` list. In a build session, the
+    # except_ elements are excluded from the tracking plan.
+    #
+    # If a session_name is provided, we treat the block as a session, and print
+    # the session header and summary, and time the main session from startup time.
     #
     @contextmanager
-    def partially_initialized(self, *, fetch_subprojects=False):
+    def initialized(self, *, session_name=None):
         directory = self._main_options['directory']
         config = self._main_options['config']
 
+        self._session_name = session_name
+
+        #
+        # Load the Context
+        #
         try:
-            self.context = Context(fetch_subprojects=fetch_subprojects)
+            self.context = Context()
             self.context.load(config)
         except BstError as e:
             self._error_exit(e, "Error loading user configuration")
@@ -170,30 +199,9 @@ class App():
             if option_value is not None:
                 setattr(self.context, context_attr, option_value)
 
-        # Create the logger right before setting the message handler
-        self.logger = LogLine(
-            self._content_profile,
-            self._format_profile,
-            self._success_profile,
-            self._error_profile,
-            self._detail_profile,
-            # Indentation for detailed messages
-            indent=INDENT,
-            # Number of last lines in an element's log to print (when encountering errors)
-            log_lines=self.context.log_error_lines,
-            # Maximum number of lines to print in a detailed message
-            message_lines=self.context.log_message_lines,
-            # Whether to print additional debugging information
-            debug=self.context.log_debug,
-            message_format=self.context.log_message_format)
-
-        # Propagate pipeline feedback to the user
-        self.context.set_message_handler(self._message_handler)
-
-        # Now that we have a logger and message handler,
-        # we can override the global exception hook.
-        sys.excepthook = self._global_exception_handler
-
+        #
+        # Load the Project
+        #
         try:
             self.project = Project(directory, self.context, cli_options=self._main_options['option'])
         except LoadError as e:
@@ -210,121 +218,65 @@ class App():
         except BstError as e:
             self._error_exit(e, "Error loading project")
 
+        # Create the logger right before setting the message handler
+        self.logger = LogLine(self.context,
+                              self._content_profile,
+                              self._format_profile,
+                              self._success_profile,
+                              self._error_profile,
+                              self._detail_profile,
+                              indent=INDENT)
+
+        # Propagate pipeline feedback to the user
+        self.context.set_message_handler(self._message_handler)
+
+        # Now that we have a logger and message handler,
+        # we can override the global exception hook.
+        sys.excepthook = self._global_exception_handler
+
+        # Create the stream right away, we'll need to pass it around
+        self.stream = Stream(self.context, self.project, self._session_start,
+                             session_start_callback=self.session_start_cb,
+                             interrupt_callback=self._interrupt_handler,
+                             ticker_callback=self._tick,
+                             job_start_callback=self._job_started,
+                             job_complete_callback=self._job_completed)
+
+        # Create our status printer, only available in interactive
+        self._status = Status(self.context,
+                              self._content_profile, self._format_profile,
+                              self._success_profile, self._error_profile,
+                              self.stream, colors=self.colors)
+
+        # Mark the beginning of the session
+        if session_name:
+            self._message(MessageType.START, session_name)
+
         # Run the body of the session here, once everything is loaded
         try:
             yield
         except BstError as e:
-            self._error_exit(e)
 
-    # initialized()
-    #
-    # Context manager to initialize the application and optionally run a session
-    # within the context manager.
-    #
-    # This context manager will take care of catching errors from within the
-    # context and report them consistently, so the CLI need not take care of
-    # reporting the errors and exiting with a consistent error status.
-    #
-    # Args:
-    #    elements (list of elements): The elements to load recursively
-    #    session_name (str): The name of the session, or None for no session
-    #    except_ (list of elements): The elements to except
-    #    rewritable (bool): Whether we should load the YAML files for roundtripping
-    #    use_configured_remote_caches (bool): Whether we should contact remotes
-    #    add_remote_cache (str): The URL for an explicitly mentioned remote cache
-    #    track_elements (list of elements): Elements which are to be tracked
-    #    track_cross_junctions (bool): Whether tracking is allowed to cross junction boundaries
-    #    track_selection (PipelineSelection): The selection algorithm for track elements
-    #    fetch_subprojects (bool): Whether we should fetch subprojects as a part of the
-    #                              loading process, if they are not yet locally cached
-    #
-    # Note that the except_ argument may have a subtly different meaning depending
-    # on the activity performed on the Pipeline. In normal circumstances the except_
-    # argument excludes elements from the `elements` list. In a build session, the
-    # except_ elements are excluded from the tracking plan.
-    #
-    # If a session_name is provided, we treat the block as a session, and print
-    # the session header and summary, and time the main session from startup time.
-    #
-    @contextmanager
-    def initialized(self, elements, *, session_name=None,
-                    except_=tuple(), rewritable=False,
-                    use_configured_remote_caches=False,
-                    add_remote_cache=None,
-                    track_elements=None,
-                    track_cross_junctions=False,
-                    track_selection=PipelineSelection.ALL,
-                    fetch_subprojects=False):
-        profile_start(Topics.LOAD_PIPELINE, "_".join(t.replace(os.sep, '-') for t in elements))
-
-        # Start with the early stage init, this enables logging right away
-        with self.partially_initialized(fetch_subprojects=fetch_subprojects):
-
-            # Mark the beginning of the session
+            # Print a nice summary if this is a session
             if session_name:
-                self._message(MessageType.START, session_name)
+                elapsed = self.stream.elapsed_time
 
-            # Create the application's scheduler
-            self.scheduler = Scheduler(self.context, self._session_start,
-                                       interrupt_callback=self._interrupt_handler,
-                                       ticker_callback=self._tick,
-                                       job_start_callback=self._job_started,
-                                       job_complete_callback=self._job_completed)
+                if isinstance(e, StreamError) and e.terminated:  # pylint: disable=no-member
+                    self._message(MessageType.WARN, session_name + ' Terminated', elapsed=elapsed)
+                else:
+                    self._message(MessageType.FAIL, session_name, elapsed=elapsed)
 
-            try:
-                self.pipeline = Pipeline(self.context, self.project, elements, except_,
-                                         rewritable=rewritable)
-            except BstError as e:
-                self._error_exit(e, "Error loading pipeline")
-
-            # Create our status printer, only available in interactive
-            self._status = Status(self._content_profile, self._format_profile,
-                                  self._success_profile, self._error_profile,
-                                  self.pipeline, self.scheduler,
-                                  colors=self.colors)
-
-            # Initialize pipeline
-            try:
-                self.pipeline.initialize(use_configured_remote_caches=use_configured_remote_caches,
-                                         add_remote_cache=add_remote_cache,
-                                         track_elements=track_elements,
-                                         track_cross_junctions=track_cross_junctions,
-                                         track_selection=track_selection)
-            except BstError as e:
-                self._error_exit(e, "Error initializing pipeline")
-
-            # Pipeline is loaded, now we can tell the logger about it
-            self.logger.size_request(self.pipeline)
-
-            profile_end(Topics.LOAD_PIPELINE, "_".join(t.replace(os.sep, '-') for t in elements))
-
-            # Print the heading
-            if session_name:
-                self._print_heading()
-
-            # Run the body of the session here, once everything is loaded
-            try:
-                yield
-            except BstError as e:
-
-                # Catch the error and summarize what happened
-                elapsed = self.scheduler.elapsed_time()
-
-                if session_name:
-                    if isinstance(e, PipelineError) and e.terminated:  # pylint: disable=no-member
-                        self._message(MessageType.WARN, session_name + ' Terminated', elapsed=elapsed)
-                    else:
-                        self._message(MessageType.FAIL, session_name, elapsed=elapsed)
-
-                if session_name:
+                if self._started:
                     self._print_summary()
 
-                # Let the outer context manager print the error and exit
-                raise
-            else:
-                # No exceptions occurred, print session time and summary
-                if session_name:
-                    self._message(MessageType.SUCCESS, session_name, elapsed=self.scheduler.elapsed_time())
+            # Exit with the error
+            self._error_exit(e)
+
+        else:
+            # No exceptions occurred, print session time and summary
+            if session_name:
+                self._message(MessageType.SUCCESS, session_name, elapsed=self.stream.elapsed_time)
+                if self._started:
                     self._print_summary()
 
     # init_project()
@@ -336,13 +288,14 @@ class App():
     # Args:
     #    project_name (str): The project name, must be a valid symbol name
     #    format_version (int): The project format version, default is the latest version
-    #    element_directory (str): The subdirectory to store elements in, default is 'elements'
+    #    element_path (str): The subdirectory to store elements in, default is 'elements'
     #    force (bool): Allow overwriting an existing project.conf
     #
     def init_project(self, project_name, format_version=BST_FORMAT_VERSION, element_path='elements', force=False):
         directory = self._main_options['directory']
         directory = os.path.abspath(directory)
         project_path = os.path.join(directory, 'project.conf')
+        elements_path = os.path.join(directory, element_path)
 
         try:
             # Abort if the project.conf already exists, unless `--force` was specified in `bst init`
@@ -371,6 +324,13 @@ class App():
             except IOError as e:
                 raise AppError("Error creating project directory {}: {}".format(directory, e)) from e
 
+            # Create the elements sub-directory if it doesnt exist
+            try:
+                os.makedirs(elements_path, exist_ok=True)
+            except IOError as e:
+                raise AppError("Error creating elements sub-directory {}: {}"
+                               .format(elements_path, e)) from e
+
             # Dont use ruamel.yaml here, because it doesnt let
             # us programatically insert comments or whitespace at
             # the toplevel.
@@ -392,22 +352,18 @@ class App():
         click.echo("Created project.conf at: {}".format(project_path), err=True)
         sys.exit(0)
 
-    # shell()
+    # shell_prompt():
     #
-    # Run a shell
+    # Creates a prompt for a shell environment, using ANSI color codes
+    # if they are available in the execution context.
     #
     # Args:
-    #    element (Element): An Element object to run the shell for
-    #    scope (Scope): The scope for the shell (Scope.BUILD or Scope.RUN)
-    #    directory (str): A directory where an existing prestaged sysroot is expected, or None
-    #    mounts (list of HostMount): Additional directories to mount into the sandbox
-    #    isolate (bool): Whether to isolate the environment like we do in builds
-    #    command (list): An argv to launch in the sandbox, or None
+    #    element (Element): The Element object to resolve a prompt for
     #
     # Returns:
-    #    (int): The exit code of the launched shell
+    #    (str): The formatted prompt to display in the shell
     #
-    def shell(self, element, scope, directory, *, mounts=None, isolate=False, command=None):
+    def shell_prompt(self, element):
         _, key, dim = element._get_display_key()
         element_name = element._get_full_name()
 
@@ -422,7 +378,7 @@ class App():
         else:
             prompt = '[{}@{}:${{PWD}}]$ '.format(key, element_name)
 
-        return element._shell(scope, directory, mounts=mounts, isolate=isolate, prompt=prompt, command=command)
+        return prompt
 
     # cleanup()
     #
@@ -431,118 +387,26 @@ class App():
     # This is called by Click at exit time
     #
     def cleanup(self):
-        if self.pipeline:
-            self.pipeline.cleanup()
+        if self.stream:
+            self.stream.cleanup()
 
     ############################################################
-    #                   Workspace Commands                     #
+    #                   Abstract Class Methods                 #
     ############################################################
 
-    # open_workspace
+    # notify()
     #
-    # Open a project workspace - this requires full initialization
+    # Notify the user of something which occurred, this
+    # is intended to grab attention from the user.
+    #
+    # This is guaranteed to only be called in interactive mode
     #
     # Args:
-    #    target (Element): The element to open the workspace for
-    #    directory (str): The directory to stage the source in
-    #    no_checkout (bool): Whether to skip checking out the source
-    #    track_first (bool): Whether to track and fetch first
-    #    force (bool): Whether to ignore contents in an existing directory
+    #    title (str): The notification title
+    #    text (str): The notification text
     #
-    def open_workspace(self, target, directory, no_checkout, track_first, force):
-
-        workdir = os.path.abspath(directory)
-
-        if not list(target.sources()):
-            build_depends = [x.name for x in target.dependencies(Scope.BUILD, recurse=False)]
-            if not build_depends:
-                raise AppError("The given element has no sources")
-            detail = "Try opening a workspace on one of its dependencies instead:\n"
-            detail += "  \n".join(build_depends)
-            raise AppError("The given element has no sources", detail=detail)
-
-        # Check for workspace config
-        if self.project.workspaces.get_workspace(target):
-            raise AppError("Workspace '{}' is already defined.".format(target.name))
-
-        # If we're going to checkout, we need at least a fetch,
-        # if we were asked to track first, we're going to fetch anyway.
-        if not no_checkout or track_first:
-            self.pipeline.fetch(self.scheduler, [target])
-
-        if not no_checkout and target._get_consistency() != Consistency.CACHED:
-            raise PipelineError("Could not stage uncached source. " +
-                                "Use `--track` to track and " +
-                                "fetch the latest version of the " +
-                                "source.")
-
-        try:
-            os.makedirs(directory, exist_ok=True)
-        except OSError as e:
-            raise AppError("Failed to create workspace directory: {}".format(e)) from e
-
-        self.project.workspaces.create_workspace(target.name, workdir)
-
-        if not no_checkout:
-            with target.timed_activity("Staging sources to {}".format(directory)):
-                target._open_workspace()
-
-        self.project.workspaces.save_config()
-        self._message(MessageType.INFO, "Saved workspace configuration")
-
-    # close_workspace
-    #
-    # Close a project workspace - this requires only partial initialization
-    #
-    # Args:
-    #    element_name (str): The element name to close the workspace for
-    #    remove_dir (bool): Whether to remove the associated directory
-    #
-    def close_workspace(self, element_name, remove_dir):
-
-        workspace = self.project.workspaces.get_workspace(element_name)
-
-        if workspace is None:
-            raise AppError("Workspace '{}' does not exist".format(element_name))
-
-        if self.interactive and remove_dir:
-            if not click.confirm('This will remove all your changes, are you sure?'):
-                click.echo('Aborting', err=True)
-                sys.exit(-1)
-
-        # Remove workspace directory if prompted
-        if remove_dir:
-            with self.context.timed_activity("Removing workspace directory {}"
-                                             .format(workspace.path)):
-                try:
-                    shutil.rmtree(workspace.path)
-                except OSError as e:
-                    raise AppError("Could not remove  '{}': {}"
-                                   .format(workspace.path, e)) from e
-
-        # Delete the workspace and save the configuration
-        self.project.workspaces.delete_workspace(element_name)
-        self.project.workspaces.save_config()
-        self._message(MessageType.INFO, "Saved workspace configuration")
-
-    # reset_workspace
-    #
-    # Reset a workspace to its original state, discarding any user
-    # changes.
-    #
-    # Args:
-    #    target (Element): The element to reset the workspace for
-    #    track (bool): Whether to also track the source
-    #
-    def reset_workspace(self, target, track):
-        workspace = self.project.workspaces.get_workspace(target.name)
-
-        if workspace is None:
-            raise AppError("Workspace '{}' is currently not defined"
-                           .format(target.name))
-
-        self.close_workspace(target.name, True)
-        self.open_workspace(target, workspace.path, False, track, False)
+    def notify(self, title, text):
+        pass
 
     ############################################################
     #                      Local Functions                     #
@@ -566,8 +430,8 @@ class App():
 
         # If the scheduler has started, try to terminate all jobs gracefully,
         # otherwise exit immediately.
-        if self.scheduler.loop:
-            self.scheduler.terminate_jobs()
+        if self.stream.running:
+            self.stream.terminate()
         else:
             sys.exit(-1)
 
@@ -577,8 +441,8 @@ class App():
     def _maybe_render_status(self):
 
         # If we're suspended or terminating, then dont render the status area
-        if self._status and self.scheduler and \
-           not (self.scheduler.suspended or self.scheduler.terminated):
+        if self._status and self.stream and \
+           not (self.stream.suspended or self.stream.terminated):
             self._status.render()
 
     #
@@ -589,7 +453,7 @@ class App():
         # Only handle ^C interactively in interactive mode
         if not self.interactive:
             self._status.clear()
-            self.scheduler.terminate_jobs()
+            self.stream.terminate()
             return
 
         # Here we can give the user some choices, like whether they would
@@ -618,41 +482,48 @@ class App():
 
             if choice == 'terminate':
                 click.echo("\nTerminating all jobs at user request\n", err=True)
-                self.scheduler.terminate_jobs()
+                self.stream.terminate()
             else:
                 if choice == 'quit':
                     click.echo("\nCompleting ongoing tasks before quitting\n", err=True)
-                    self.scheduler.stop_queueing()
+                    self.stream.quit()
                 elif choice == 'continue':
                     click.echo("\nContinuing\n", err=True)
 
     def _tick(self, elapsed):
         self._maybe_render_status()
 
-    def _job_started(self, element, action_name):
-        self._status.add_job(element, action_name)
+    def _job_started(self, job):
+        self._status.add_job(job)
         self._maybe_render_status()
 
-    def _job_completed(self, element, queue, action_name, success):
-        self._status.remove_job(element, action_name)
+    def _job_completed(self, job, success):
+        self._status.remove_job(job)
         self._maybe_render_status()
 
         # Dont attempt to handle a failure if the user has already opted to
         # terminate
-        if not success and not self.scheduler.terminated:
+        if not success and not self.stream.terminated:
 
-            # Get the last failure message for additional context
-            failure = self._fail_messages.get(element._get_unique_id())
+            if isinstance(job, ElementJob):
+                element = job.element
+                queue = job.queue
 
-            # XXX This is dangerous, sometimes we get the job completed *before*
-            # the failure message reaches us ??
-            if not failure:
-                self._status.clear()
-                click.echo("\n\n\nBUG: Message handling out of sync, " +
-                           "unable to retrieve failure message for element {}\n\n\n\n\n"
-                           .format(element), err=True)
+                # Get the last failure message for additional context
+                failure = self._fail_messages.get(element._get_unique_id())
+
+                # XXX This is dangerous, sometimes we get the job completed *before*
+                # the failure message reaches us ??
+                if not failure:
+                    self._status.clear()
+                    click.echo("\n\n\nBUG: Message handling out of sync, " +
+                               "unable to retrieve failure message for element {}\n\n\n\n\n"
+                               .format(element), err=True)
+                else:
+                    self._handle_failure(element, queue, failure)
             else:
-                self._handle_failure(element, queue, failure)
+                click.echo("\nTerminating all jobs\n", err=True)
+                self.stream.terminate()
 
     def _handle_failure(self, element, queue, failure):
 
@@ -660,9 +531,9 @@ class App():
         if not self._interactive_failures:
 
             if self.context.sched_error_action == 'terminate':
-                self.scheduler.terminate_jobs()
+                self.stream.terminate()
             elif self.context.sched_error_action == 'quit':
-                self.scheduler.stop_queueing()
+                self.stream.quit()
             elif self.context.sched_error_action == 'continue':
                 pass
             return
@@ -693,6 +564,9 @@ class App():
             while choice not in ['continue', 'quit', 'terminate', 'retry']:
                 click.echo(summary, err=True)
 
+                self.notify("BuildStream failure", "{} on element {}"
+                            .format(failure.action_name, element.name))
+
                 try:
                     choice = click.prompt("Choice:", default='continue', err=True,
                                           value_proc=_prefix_choice_value_proc(choices))
@@ -706,7 +580,8 @@ class App():
                 if choice == 'shell':
                     click.echo("\nDropping into an interactive shell in the failed build sandbox\n", err=True)
                     try:
-                        self.shell(element, Scope.BUILD, failure.sandbox, isolate=True)
+                        prompt = self.shell_prompt(element)
+                        self.stream.shell(element, Scope.BUILD, prompt, directory=failure.sandbox, isolate=True)
                     except BstError as e:
                         click.echo("Error while attempting to create interactive shell: {}".format(e), err=True)
                 elif choice == 'log':
@@ -716,11 +591,11 @@ class App():
 
             if choice == 'terminate':
                 click.echo("\nTerminating all jobs\n", err=True)
-                self.scheduler.terminate_jobs()
+                self.stream.terminate()
             else:
                 if choice == 'quit':
                     click.echo("\nCompleting ongoing tasks before quitting\n", err=True)
-                    self.scheduler.stop_queueing()
+                    self.stream.quit()
                 elif choice == 'continue':
                     click.echo("\nContinuing with other non failing elements\n", err=True)
                 elif choice == 'retry':
@@ -729,21 +604,23 @@ class App():
                     queue.enqueue([element])
 
     #
-    # Prints the application startup heading, used for commands which
-    # will process a pipeline.
+    # Print the session heading if we've loaded a pipeline and there
+    # is going to be a session
     #
-    def _print_heading(self, deps=None):
-        self.logger.print_heading(self.pipeline,
-                                  self._main_options['log_file'],
-                                  styling=self.colors,
-                                  deps=deps)
+    def session_start_cb(self):
+        self._started = True
+        if self._session_name:
+            self.logger.print_heading(self.project,
+                                      self.stream,
+                                      log_file=self._main_options['log_file'],
+                                      styling=self.colors)
 
     #
     # Print a summary of the queues
     #
     def _print_summary(self):
         click.echo("", err=True)
-        self.logger.print_summary(self.pipeline, self.scheduler,
+        self.logger.print_summary(self.stream,
                                   self._main_options['log_file'],
                                   styling=self.colors)
 
@@ -807,7 +684,7 @@ class App():
     def _interrupted(self):
         self._status.clear()
         try:
-            with self.scheduler.jobs_suspended():
+            with self.stream.suspend():
                 yield
         finally:
             self._maybe_render_status()
