@@ -3,7 +3,7 @@ import sys
 
 import click
 from .. import _yaml
-from .._exceptions import BstError, LoadError
+from .._exceptions import BstError, LoadError, AppError
 from .._versions import BST_FORMAT_VERSION
 from .complete import main_bashcomplete, complete_path, CompleteUnhandled
 
@@ -12,6 +12,45 @@ from .complete import main_bashcomplete, complete_path, CompleteUnhandled
 #            Override of click's main entry point                #
 ##################################################################
 
+# search_command()
+#
+# Helper function to get a command and context object
+# for a given command.
+#
+# Args:
+#    commands (list): A list of command words following `bst` invocation
+#    context (click.Context): An existing toplevel context, or None
+#
+# Returns:
+#    context (click.Context): The context of the associated command, or None
+#
+def search_command(args, *, context=None):
+    if context is None:
+        context = cli.make_context('bst', args, resilient_parsing=True)
+
+    # Loop into the deepest command
+    command = cli
+    command_ctx = context
+    for cmd in args:
+        command = command_ctx.command.get_command(command_ctx, cmd)
+        if command is None:
+            return None
+        command_ctx = command.make_context(command.name, [command.name],
+                                           parent=command_ctx,
+                                           resilient_parsing=True)
+
+    return command_ctx
+
+
+# Completion for completing command names as help arguments
+def complete_commands(cmd, args, incomplete):
+    command_ctx = search_command(args[1:])
+    if command_ctx and command_ctx.command and isinstance(command_ctx.command, click.MultiCommand):
+        return [subcommand + " " for subcommand in command_ctx.command.list_commands(command_ctx)]
+
+    return []
+
+
 # Special completion for completing the bst elements in a project dir
 def complete_target(args, incomplete):
     """
@@ -20,6 +59,18 @@ def complete_target(args, incomplete):
     :return: all the possible user-specified completions for the param
     """
 
+    project_conf = 'project.conf'
+
+    def ensure_project_dir(directory):
+        directory = os.path.abspath(directory)
+        while not os.path.isfile(os.path.join(directory, project_conf)):
+            parent_dir = os.path.dirname(directory)
+            if directory == parent_dir:
+                break
+            directory = parent_dir
+
+        return directory
+
     # First resolve the directory, in case there is an
     # active --directory/-C option
     #
@@ -35,10 +86,14 @@ def complete_target(args, incomplete):
 
     if idx >= 0 and len(args) > idx + 1:
         base_directory = args[idx + 1]
+    else:
+        # Check if this directory or any of its parent directories
+        # contain a project config file
+        base_directory = ensure_project_dir(base_directory)
 
     # Now parse the project.conf just to find the element path,
     # this is unfortunately a bit heavy.
-    project_file = os.path.join(base_directory, 'project.conf')
+    project_file = os.path.join(base_directory, project_conf)
     try:
         project = _yaml.load(project_file)
     except LoadError:
@@ -57,7 +112,7 @@ def complete_target(args, incomplete):
     return complete_path("File", incomplete, base_directory=base_directory)
 
 
-def override_completions(cmd_param, args, incomplete):
+def override_completions(cmd, cmd_param, args, incomplete):
     """
     :param cmd_param: command definition
     :param args: full list of args typed before the incomplete arg
@@ -65,6 +120,9 @@ def override_completions(cmd_param, args, incomplete):
     :return: all the possible user-specified completions for the param
     """
 
+    if cmd.name == 'help':
+        return complete_commands(cmd, args, incomplete)
+
     # We can't easily extend click's data structures without
     # modifying click itself, so just do some weak special casing
     # right here and select which parameters we want to handle specially.
@@ -170,10 +228,37 @@ def cli(context, **kwargs):
     from .app import App
 
     # Create the App, giving it the main arguments
-    context.obj = App(dict(kwargs))
+    context.obj = App.create(dict(kwargs))
     context.call_on_close(context.obj.cleanup)
 
 
+##################################################################
+#                           Help Command                         #
+##################################################################
+@cli.command(name="help", short_help="Print usage information",
+             context_settings={"help_option_names": []})
+@click.argument("command", nargs=-1, metavar='COMMAND')
+@click.pass_context
+def help_command(ctx, command):
+    """Print usage information about a given command
+    """
+    command_ctx = search_command(command, context=ctx.parent)
+    if not command_ctx:
+        click.echo("Not a valid command: '{} {}'"
+                   .format(ctx.parent.info_name, " ".join(command)), err=True)
+        sys.exit(-1)
+
+    click.echo(command_ctx.command.get_help(command_ctx), err=True)
+
+    # Hint about available sub commands
+    if isinstance(command_ctx.command, click.MultiCommand):
+        detail = " "
+        if command:
+            detail = " {} ".format(" ".join(command))
+        click.echo("\nFor usage on a specific command: {} help{}COMMAND"
+                   .format(ctx.parent.info_name, detail), err=True)
+
+
 ##################################################################
 #                           Init Command                         #
 ##################################################################
@@ -206,20 +291,20 @@ def init(app, project_name, format_version, element_path, force):
 @click.option('--all', 'all_', default=False, is_flag=True,
               help="Build elements that would not be needed for the current build plan")
 @click.option('--track', 'track_', multiple=True,
-              type=click.Path(dir_okay=False, readable=True),
+              type=click.Path(readable=False),
               help="Specify elements to track during the build. Can be used "
                    "repeatedly to specify multiple elements")
 @click.option('--track-all', default=False, is_flag=True,
               help="Track all elements in the pipeline")
 @click.option('--track-except', multiple=True,
-              type=click.Path(dir_okay=False, readable=True),
+              type=click.Path(readable=False),
               help="Except certain dependencies from tracking")
 @click.option('--track-cross-junctions', '-J', default=False, is_flag=True,
               help="Allow tracking to cross junction boundaries")
 @click.option('--track-save', default=False, is_flag=True,
               help="Deprecated: This is ignored")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
 def build(app, elements, all_, track_, track_save, track_all, track_except, track_cross_junctions):
     """Build elements in a pipeline"""
@@ -235,15 +320,12 @@ def build(app, elements, all_, track_, track_save, track_all, track_except, trac
     if track_all:
         track_ = elements
 
-    rewritable = False
-    if track_:
-        rewritable = True
-
-    with app.initialized(elements, session_name="Build", except_=track_except, rewritable=rewritable,
-                         use_configured_remote_caches=True, track_elements=track_,
+    with app.initialized(session_name="Build"):
+        app.stream.build(elements,
+                         track_targets=track_,
+                         track_except=track_except,
                          track_cross_junctions=track_cross_junctions,
-                         fetch_subprojects=True):
-        app.pipeline.build(app.scheduler, build_all=all_)
+                         build_all=all_)
 
 
 ##################################################################
@@ -251,7 +333,7 @@ def build(app, elements, all_, track_, track_save, track_all, track_except, trac
 ##################################################################
 @cli.command(short_help="Fetch sources in a pipeline")
 @click.option('--except', 'except_', multiple=True,
-              type=click.Path(dir_okay=False, readable=True),
+              type=click.Path(readable=False),
               help="Except certain dependencies from fetching")
 @click.option('--deps', '-d', default='plan',
               type=click.Choice(['none', 'plan', 'all']),
@@ -261,7 +343,7 @@ def build(app, elements, all_, track_, track_save, track_all, track_except, trac
 @click.option('--track-cross-junctions', '-J', default=False, is_flag=True,
               help="Allow tracking to cross junction boundaries")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
 def fetch(app, elements, deps, track_, except_, track_cross_junctions):
     """Fetch sources required to build the pipeline
@@ -278,16 +360,23 @@ def fetch(app, elements, deps, track_, except_, track_cross_junctions):
         plan:  Only dependencies required for the build plan
         all:   All dependencies
     """
+    from .._pipeline import PipelineSelection
+
     if track_cross_junctions and not track_:
         click.echo("ERROR: The --track-cross-junctions option can only be used with --track", err=True)
         sys.exit(-1)
 
-    with app.initialized(elements, session_name="Fetch", except_=except_, rewritable=track_,
-                         track_elements=elements if track_ else None,
-                         track_cross_junctions=track_cross_junctions,
-                         fetch_subprojects=True):
-        dependencies = app.pipeline.get_selection(deps)
-        app.pipeline.fetch(app.scheduler, dependencies)
+    if track_ and deps == PipelineSelection.PLAN:
+        click.echo("WARNING: --track specified for tracking of a build plan\n\n"
+                   "Since tracking modifies the build plan, all elements will be tracked.", err=True)
+        deps = PipelineSelection.ALL
+
+    with app.initialized(session_name="Fetch"):
+        app.stream.fetch(elements,
+                         selection=deps,
+                         except_targets=except_,
+                         track_targets=track_,
+                         track_cross_junctions=track_cross_junctions)
 
 
 ##################################################################
@@ -295,7 +384,7 @@ def fetch(app, elements, deps, track_, except_, track_cross_junctions):
 ##################################################################
 @cli.command(short_help="Track new source references")
 @click.option('--except', 'except_', multiple=True,
-              type=click.Path(dir_okay=False, readable=True),
+              type=click.Path(readable=False),
               help="Except certain dependencies from tracking")
 @click.option('--deps', '-d', default='none',
               type=click.Choice(['none', 'all']),
@@ -303,7 +392,7 @@ def fetch(app, elements, deps, track_, except_, track_cross_junctions):
 @click.option('--cross-junctions', '-J', default=False, is_flag=True,
               help="Allow crossing junction boundaries")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
 def track(app, elements, deps, except_, cross_junctions):
     """Consults the specified tracking branches for new versions available
@@ -318,12 +407,15 @@ def track(app, elements, deps, except_, cross_junctions):
         none:  No dependencies, just the specified elements
         all:   All dependencies of all specified elements
     """
-    with app.initialized(elements, session_name="Track", except_=except_, rewritable=True,
-                         track_elements=elements,
-                         track_cross_junctions=cross_junctions,
-                         track_selection=deps,
-                         fetch_subprojects=True):
-        app.pipeline.track(app.scheduler)
+    with app.initialized(session_name="Track"):
+        # Substitute 'none' for 'redirect' so that element redirections
+        # will be done
+        if deps == 'none':
+            deps = 'redirect'
+        app.stream.track(elements,
+                         selection=deps,
+                         except_targets=except_,
+                         cross_junctions=cross_junctions)
 
 
 ##################################################################
@@ -336,7 +428,7 @@ def track(app, elements, deps, except_, cross_junctions):
 @click.option('--remote', '-r',
               help="The URL of the remote cache (defaults to the first configured cache)")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
 def pull(app, elements, deps, remote):
     """Pull a built artifact from the configured remote artifact cache.
@@ -351,10 +443,8 @@ def pull(app, elements, deps, remote):
         none:  No dependencies, just the element itself
         all:   All dependencies
     """
-    with app.initialized(elements, session_name="Pull", use_configured_remote_caches=(remote is None),
-                         add_remote_cache=remote, fetch_subprojects=True):
-        to_pull = app.pipeline.get_selection(deps)
-        app.pipeline.pull(app.scheduler, to_pull)
+    with app.initialized(session_name="Pull"):
+        app.stream.pull(elements, selection=deps, remote=remote)
 
 
 ##################################################################
@@ -367,7 +457,7 @@ def pull(app, elements, deps, remote):
 @click.option('--remote', '-r', default=None,
               help="The URL of the remote cache (defaults to the first configured cache)")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
 def push(app, elements, deps, remote):
     """Push a built artifact to a remote artifact cache.
@@ -381,11 +471,8 @@ def push(app, elements, deps, remote):
         none:  No dependencies, just the element itself
         all:   All dependencies
     """
-    with app.initialized(elements, session_name="Push",
-                         use_configured_remote_caches=(remote is None),
-                         add_remote_cache=remote, fetch_subprojects=True):
-        to_push = app.pipeline.get_selection(deps)
-        app.pipeline.push(app.scheduler, to_push)
+    with app.initialized(session_name="Push"):
+        app.stream.push(elements, selection=deps, remote=remote)
 
 
 ##################################################################
@@ -393,7 +480,7 @@ def push(app, elements, deps, remote):
 ##################################################################
 @cli.command(short_help="Show elements in the pipeline")
 @click.option('--except', 'except_', multiple=True,
-              type=click.Path(dir_okay=False, readable=True),
+              type=click.Path(readable=False),
               help="Except certain dependencies")
 @click.option('--deps', '-d', default='all',
               type=click.Choice(['none', 'plan', 'run', 'build', 'all']),
@@ -404,12 +491,10 @@ def push(app, elements, deps, remote):
 @click.option('--format', '-f', 'format_', metavar='FORMAT', default=None,
               type=click.STRING,
               help='Format string for each element')
-@click.option('--downloadable', default=False, is_flag=True,
-              help="Refresh downloadable state")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
-def show(app, elements, deps, except_, order, format_, downloadable):
+def show(app, elements, deps, except_, order, format_):
     """Show elements in the pipeline
 
     By default this will show all of the dependencies of the
@@ -456,9 +541,11 @@ def show(app, elements, deps, except_, order, format_, downloadable):
         bst show target.bst --format \\
             $'---------- %{name} ----------\\n%{vars}'
     """
-    with app.initialized(elements, except_=except_, use_configured_remote_caches=downloadable):
+    with app.initialized():
+        dependencies = app.stream.load_selection(elements,
+                                                 selection=deps,
+                                                 except_targets=except_)
 
-        dependencies = app.pipeline.get_selection(deps)
         if order == "alpha":
             dependencies = sorted(dependencies)
 
@@ -466,8 +553,7 @@ def show(app, elements, deps, except_, order, format_, downloadable):
             format_ = app.context.log_element_format
 
         report = app.logger.show_pipeline(dependencies, format_)
-
-    click.echo(report, color=app.colors)
+        click.echo(report, color=app.colors)
 
 
 ##################################################################
@@ -485,7 +571,7 @@ def show(app, elements, deps, except_, order, format_, downloadable):
 @click.option('--isolate', is_flag=True, default=False,
               help='Create an isolated build sandbox')
 @click.argument('element',
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.argument('command', type=click.STRING, nargs=-1)
 @click.pass_obj
 def shell(app, element, sysroot, mount, isolate, build_, command):
@@ -507,43 +593,32 @@ def shell(app, element, sysroot, mount, isolate, build_, command):
     """
     from ..element import Scope
     from .._project import HostMount
+    from .._pipeline import PipelineSelection
+
     if build_:
         scope = Scope.BUILD
     else:
         scope = Scope.RUN
 
-    with app.initialized((element,)):
-        pass
+    with app.initialized():
+        dependencies = app.stream.load_selection((element,), selection=PipelineSelection.NONE)
+        element = dependencies[0]
+        prompt = app.shell_prompt(element)
+        mounts = [
+            HostMount(path, host_path)
+            for host_path, path in mount
+        ]
+        try:
+            exitcode = app.stream.shell(element, scope, prompt,
+                                        directory=sysroot,
+                                        mounts=mounts,
+                                        isolate=isolate,
+                                        command=command)
+        except BstError as e:
+            raise AppError("Error launching shell: {}".format(e), detail=e.detail) from e
 
-    # Assert we have everything we need built.
-    missing_deps = []
-    if scope is not None:
-        for dep in app.pipeline.dependencies(scope):
-            if not dep._cached():
-                missing_deps.append(dep)
-
-    if missing_deps:
-        click.echo("", err=True)
-        click.echo("Missing elements for staging an environment for a shell:", err=True)
-        for dep in missing_deps:
-            click.echo("   {}".format(dep.name), err=True)
-        click.echo("", err=True)
-        click.echo("Try building them first", err=True)
-        sys.exit(-1)
-
-    mounts = [
-        HostMount(path, host_path)
-        for host_path, path in mount
-    ]
-
-    try:
-        element = app.pipeline.targets[0]
-        exitcode = app.shell(element, scope, sysroot, mounts=mounts, isolate=isolate, command=command)
-        sys.exit(exitcode)
-    except BstError as e:
-        click.echo("", err=True)
-        click.echo("Errors shelling into this pipeline: {}".format(e), err=True)
-        sys.exit(-1)
+    # If there were no errors, we return the shell's exit code here.
+    sys.exit(exitcode)
 
 
 ##################################################################
@@ -552,48 +627,27 @@ def shell(app, element, sysroot, mount, isolate, build_, command):
 @cli.command(short_help="Checkout a built artifact")
 @click.option('--force', '-f', default=False, is_flag=True,
               help="Overwrite files existing in checkout directory")
+@click.option('--deps', '-d', default='run',
+              type=click.Choice(['run', 'none']),
+              help='The dependencies to checkout (default: run)')
 @click.option('--integrate/--no-integrate', default=True, is_flag=True,
               help="Whether to run integration commands")
 @click.option('--hardlinks', default=False, is_flag=True,
               help="Checkout hardlinks instead of copies (handle with care)")
 @click.argument('element',
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.argument('directory', type=click.Path(file_okay=False))
 @click.pass_obj
-def checkout(app, element, directory, force, integrate, hardlinks):
+def checkout(app, element, directory, force, deps, integrate, hardlinks):
     """Checkout a built artifact to the specified directory
     """
-    with app.initialized((element,)):
-        app.pipeline.checkout(directory, force, integrate, hardlinks)
-
-
-##################################################################
-#                     Source Bundle Command                      #
-##################################################################
-@cli.command(name="source-bundle", short_help="Produce a build bundle to be manually executed")
-@click.option('--except', 'except_', multiple=True,
-              type=click.Path(dir_okay=False, readable=True),
-              help="Elements to except from the tarball")
-@click.option('--compression', default='gz',
-              type=click.Choice(['none', 'gz', 'bz2', 'xz']),
-              help="Compress the tar file using the given algorithm.")
-@click.option('--track', 'track_', default=False, is_flag=True,
-              help="Track new source references before building")
-@click.option('--force', '-f', default=False, is_flag=True,
-              help="Overwrite files existing in checkout directory")
-@click.option('--directory', default=os.getcwd(),
-              help="The directory to write the tarball to")
-@click.argument('target',
-                type=click.Path(dir_okay=False, readable=True))
-@click.pass_obj
-def source_bundle(app, target, force, directory,
-                  track_, compression, except_):
-    """Produce a source bundle to be manually executed
-    """
-    with app.initialized((target,), rewritable=track_, track_elements=[target] if track_ else None):
-        dependencies = app.pipeline.get_selection('all')
-        app.pipeline.source_bundle(app.scheduler, dependencies, force, track_,
-                                   compression, directory)
+    with app.initialized():
+        app.stream.checkout(element,
+                            directory=directory,
+                            deps=deps,
+                            force=force,
+                            integrate=integrate,
+                            hardlinks=hardlinks)
 
 
 ##################################################################
@@ -616,7 +670,7 @@ def workspace():
 @click.option('--track', 'track_', default=False, is_flag=True,
               help="Track and fetch new source references before checking out the workspace")
 @click.argument('element',
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.argument('directory', type=click.Path(file_okay=False))
 @click.pass_obj
 def workspace_open(app, no_checkout, force, track_, element, directory):
@@ -632,10 +686,11 @@ def workspace_open(app, no_checkout, force, track_, element, directory):
             click.echo("Checkout directory is not empty: {}".format(directory), err=True)
             sys.exit(-1)
 
-    with app.initialized((element,), rewritable=track_, track_elements=[element] if track_ else None):
-        # This command supports only one target
-        target = app.pipeline.targets[0]
-        app.open_workspace(target, directory, no_checkout, track_, force)
+    with app.initialized():
+        app.stream.workspace_open(element, directory,
+                                  no_checkout=no_checkout,
+                                  track_first=track_,
+                                  force=force)
 
 
 ##################################################################
@@ -647,7 +702,7 @@ def workspace_open(app, no_checkout, force, track_, element, directory):
 @click.option('--all', '-a', 'all_', default=False, is_flag=True,
               help="Close all open workspaces")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
 def workspace_close(app, remove_dir, all_, elements):
     """Close a workspace"""
@@ -656,43 +711,69 @@ def workspace_close(app, remove_dir, all_, elements):
         click.echo('ERROR: no elements specified', err=True)
         sys.exit(-1)
 
-    with app.partially_initialized():
+    with app.initialized():
+
+        # Early exit if we specified `all` and there are no workspaces
+        if all_ and not app.stream.workspace_exists():
+            click.echo('No open workspaces to close', err=True)
+            sys.exit(0)
+
         if all_:
-            elements = [element_name for element_name, _ in app.project.workspaces.list()]
-        for element in elements:
-            app.close_workspace(element, remove_dir)
+            elements = [element_name for element_name, _ in app.context.get_workspaces().list()]
+
+        elements = app.stream.redirect_element_names(elements)
+
+        # Check that the workspaces in question exist
+        nonexisting = []
+        for element_name in elements:
+            if not app.stream.workspace_exists(element_name):
+                nonexisting.append(element_name)
+        if nonexisting:
+            raise AppError("Workspace does not exist", detail="\n".join(nonexisting))
+
+        if app.interactive and remove_dir:
+            if not click.confirm('This will remove all your changes, are you sure?'):
+                click.echo('Aborting', err=True)
+                sys.exit(-1)
+
+        for element_name in elements:
+            app.stream.workspace_close(element_name, remove_dir=remove_dir)
 
 
 ##################################################################
 #                     Workspace Reset Command                    #
 ##################################################################
 @workspace.command(name='reset', short_help="Reset a workspace to its original state")
+@click.option('--soft', default=False, is_flag=True,
+              help="Reset workspace state without affecting its contents")
 @click.option('--track', 'track_', default=False, is_flag=True,
               help="Track and fetch the latest source before resetting")
 @click.option('--all', '-a', 'all_', default=False, is_flag=True,
               help="Reset all open workspaces")
 @click.argument('elements', nargs=-1,
-                type=click.Path(dir_okay=False, readable=True))
+                type=click.Path(readable=False))
 @click.pass_obj
-def workspace_reset(app, track_, all_, elements):
+def workspace_reset(app, soft, track_, all_, elements):
     """Reset a workspace to its original state"""
 
-    if not (all_ or elements):
-        click.echo('ERROR: no elements specified', err=True)
-        sys.exit(-1)
+    # Check that the workspaces in question exist
+    with app.initialized():
 
-    if app.interactive:
-        if not click.confirm('This will remove all your changes, are you sure?'):
-            click.echo('Aborting', err=True)
-            sys.exit(-1)
+        if not (all_ or elements):
+            raise AppError('No elements specified to reset')
+
+        if all_ and not app.stream.workspace_exists():
+            raise AppError("No open workspaces to reset")
+
+        if app.interactive and not soft:
+            if not click.confirm('This will remove all your changes, are you sure?'):
+                click.echo('Aborting', err=True)
+                sys.exit(-1)
 
-    with app.partially_initialized():
         if all_:
-            elements = tuple(element_name for element_name, _ in app.project.workspaces.list())
+            elements = tuple(element_name for element_name, _ in app.context.get_workspaces().list())
 
-    with app.initialized(elements):
-        for target in app.pipeline.targets:
-            app.reset_workspace(target, track_)
+        app.stream.workspace_reset(elements, soft=soft, track_first=track_)
 
 
 ##################################################################
@@ -703,15 +784,35 @@ def workspace_reset(app, track_, all_, elements):
 def workspace_list(app):
     """List open workspaces"""
 
-    with app.partially_initialized():
-        workspaces = []
-        for element_name, workspace_ in app.project.workspaces.list():
-            workspace_detail = {
-                'element': element_name,
-                'directory': workspace_.path,
-            }
-            workspaces.append(workspace_detail)
+    with app.initialized():
+        app.stream.workspace_list()
 
-        _yaml.dump({
-            'workspaces': workspaces
-        })
+
+##################################################################
+#                     Source Bundle Command                      #
+##################################################################
+@cli.command(name="source-bundle", short_help="Produce a build bundle to be manually executed")
+@click.option('--except', 'except_', multiple=True,
+              type=click.Path(readable=False),
+              help="Elements to except from the tarball")
+@click.option('--compression', default='gz',
+              type=click.Choice(['none', 'gz', 'bz2', 'xz']),
+              help="Compress the tar file using the given algorithm.")
+@click.option('--track', 'track_', default=False, is_flag=True,
+              help="Track new source references before bundling")
+@click.option('--force', '-f', default=False, is_flag=True,
+              help="Overwrite an existing tarball")
+@click.option('--directory', default=os.getcwd(),
+              help="The directory to write the tarball to")
+@click.argument('element',
+                type=click.Path(readable=False))
+@click.pass_obj
+def source_bundle(app, element, force, directory,
+                  track_, compression, except_):
+    """Produce a source bundle to be manually executed
+    """
+    with app.initialized():
+        app.stream.source_bundle(element, directory,
+                                 track_first=track_,
+                                 force=force,
+                                 compression=compression)
