@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2017 Codethink Limited
 #
@@ -28,7 +27,7 @@ import click
 from ruamel import yaml
 
 from . import Profile
-from .. import Element, Scope, Consistency
+from .. import Element, Consistency
 from .. import _yaml
 from .. import __version__ as bst_version
 from .._exceptions import ImplError
@@ -50,7 +49,10 @@ ERROR_MESSAGES = [MessageType.FAIL, MessageType.ERROR, MessageType.BUG]
 #
 class Widget():
 
-    def __init__(self, content_profile, format_profile):
+    def __init__(self, context, content_profile, format_profile):
+
+        # The context
+        self.context = context
 
         # The content profile
         self.content_profile = content_profile
@@ -58,17 +60,6 @@ class Widget():
         # The formatting profile
         self.format_profile = format_profile
 
-    # size_request()
-    #
-    # Gives the widget a chance to preflight the pipeline
-    # and figure out what size it might need for alignment purposes
-    #
-    # Args:
-    #    pipeline (Pipeline): The pipeline to process
-    #
-    def size_request(self, pipeline):
-        pass
-
     # render()
     #
     # Renders a string to be printed in the UI
@@ -93,8 +84,8 @@ class Space(Widget):
 # Used to add fixed text between columns
 class FixedText(Widget):
 
-    def __init__(self, text, content_profile, format_profile):
-        super(FixedText, self).__init__(content_profile, format_profile)
+    def __init__(self, context, text, content_profile, format_profile):
+        super(FixedText, self).__init__(context, content_profile, format_profile)
         self.text = text
 
     def render(self, message):
@@ -127,9 +118,9 @@ class Debug(Widget):
 
 # A widget for rendering the time codes
 class TimeCode(Widget):
-    def __init__(self, content_profile, format_profile, microseconds=False):
+    def __init__(self, context, content_profile, format_profile, microseconds=False):
         self._microseconds = microseconds
-        super(TimeCode, self).__init__(content_profile, format_profile)
+        super(TimeCode, self).__init__(context, content_profile, format_profile)
 
     def render(self, message):
         return self.render_time(message.elapsed)
@@ -183,23 +174,13 @@ class TypeName(Widget):
 # A widget for displaying the Element name
 class ElementName(Widget):
 
-    def __init__(self, content_profile, format_profile):
-        super(ElementName, self).__init__(content_profile, format_profile)
+    def __init__(self, context, content_profile, format_profile):
+        super(ElementName, self).__init__(context, content_profile, format_profile)
 
         # Pre initialization format string, before we know the length of
         # element names in the pipeline
         self._fmt_string = '{: <30}'
 
-    def size_request(self, pipeline):
-        longest_name = 0
-        for plugin in pipeline.dependencies(Scope.ALL, include_sources=True):
-            longest_name = max(len(plugin.name), longest_name)
-
-        # Put a cap at a specific width, usually some elements cause the line
-        # to be too long, just live with the unaligned columns in that case
-        longest_name = min(longest_name, 30)
-        self._fmt_string = '{: <' + str(longest_name) + '}'
-
     def render(self, message):
         element_id = message.task_id or message.unique_id
         if element_id is None:
@@ -222,21 +203,17 @@ class ElementName(Widget):
 class MessageText(Widget):
 
     def render(self, message):
-
         return message.message
 
 
 # A widget for formatting the element cache key
 class CacheKey(Widget):
 
-    def __init__(self, content_profile, format_profile, err_profile):
-        super(CacheKey, self).__init__(content_profile, format_profile)
+    def __init__(self, context, content_profile, format_profile, err_profile):
+        super(CacheKey, self).__init__(context, content_profile, format_profile)
 
         self._err_profile = err_profile
-        self._key_length = 0
-
-    def size_request(self, pipeline):
-        self._key_length = pipeline.context.log_key_length
+        self._key_length = context.log_key_length
 
     def render(self, message):
 
@@ -261,16 +238,11 @@ class CacheKey(Widget):
 # A widget for formatting the log file
 class LogFile(Widget):
 
-    def __init__(self, content_profile, format_profile, err_profile):
-        super(LogFile, self).__init__(content_profile, format_profile)
+    def __init__(self, context, content_profile, format_profile, err_profile):
+        super(LogFile, self).__init__(context, content_profile, format_profile)
 
         self._err_profile = err_profile
-        self._logdir = ''
-
-    def size_request(self, pipeline):
-
-        # Hold on to the logging directory so we can abbreviate
-        self._logdir = pipeline.context.logdir
+        self._logdir = context.logdir
 
     def render(self, message, abbrev=True):
 
@@ -296,14 +268,10 @@ class LogFile(Widget):
 # these messages, and the message text for other types.
 #
 class MessageOrLogFile(Widget):
-    def __init__(self, content_profile, format_profile, err_profile):
-        super(MessageOrLogFile, self).__init__(content_profile, format_profile)
-        self._message_widget = MessageText(content_profile, format_profile)
-        self._logfile_widget = LogFile(content_profile, format_profile, err_profile)
-
-    def size_request(self, pipeline):
-        self._message_widget.size_request(pipeline)
-        self._logfile_widget.size_request(pipeline)
+    def __init__(self, context, content_profile, format_profile, err_profile):
+        super(MessageOrLogFile, self).__init__(context, content_profile, format_profile)
+        self._message_widget = MessageText(context, content_profile, format_profile)
+        self._logfile_widget = LogFile(context, content_profile, format_profile, err_profile)
 
     def render(self, message):
         # Show the log file only in the main start/success messages
@@ -315,46 +283,58 @@ class MessageOrLogFile(Widget):
         return text
 
 
+# LogLine
+#
 # A widget for formatting a log line
+#
+# Args:
+#    context (Context): The Context
+#    content_profile (Profile): Formatting profile for content text
+#    format_profile (Profile): Formatting profile for formatting text
+#    success_profile (Profile): Formatting profile for success text
+#    error_profile (Profile): Formatting profile for error text
+#    detail_profile (Profile): Formatting profile for detail text
+#    indent (int): Number of spaces to use for general indentation
+#
 class LogLine(Widget):
 
-    def __init__(self, content_profile, format_profile, success_profile, err_profile, detail_profile,
-                 indent=4,
-                 log_lines=10,
-                 message_lines=10,
-                 debug=False,
-                 message_format: str = None):
-        super(LogLine, self).__init__(content_profile, format_profile)
+    def __init__(self, context,
+                 content_profile,
+                 format_profile,
+                 success_profile,
+                 err_profile,
+                 detail_profile,
+                 indent=4):
+        super(LogLine, self).__init__(context, content_profile, format_profile)
 
         self._columns = []
         self._failure_messages = defaultdict(list)
-        self._context = None
         self._success_profile = success_profile
         self._err_profile = err_profile
         self._detail_profile = detail_profile
         self._indent = ' ' * indent
-        self._log_lines = log_lines
-        self._message_lines = message_lines
+        self._log_lines = context.log_error_lines
+        self._message_lines = context.log_message_lines
         self._resolved_keys = None
 
-        self._space_widget = Space(content_profile, format_profile)
-        self._logfile_widget = LogFile(content_profile, format_profile, err_profile)
+        self._space_widget = Space(context, content_profile, format_profile)
+        self._logfile_widget = LogFile(context, content_profile, format_profile, err_profile)
 
-        if debug:
+        if context.log_debug:
             self._columns.extend([
-                Debug(content_profile, format_profile)
+                Debug(context, content_profile, format_profile)
             ])
 
         self.logfile_variable_names = {
-            "elapsed": TimeCode(content_profile, format_profile, microseconds=False),
-            "elapsed-us": TimeCode(content_profile, format_profile, microseconds=True),
-            "wallclock": WallclockTime(content_profile, format_profile),
-            "key": CacheKey(content_profile, format_profile, err_profile),
-            "element": ElementName(content_profile, format_profile),
-            "action": TypeName(content_profile, format_profile),
-            "message": MessageOrLogFile(content_profile, format_profile, err_profile)
+            "elapsed": TimeCode(context, content_profile, format_profile, microseconds=False),
+            "elapsed-us": TimeCode(context, content_profile, format_profile, microseconds=True),
+            "wallclock": WallclockTime(context, content_profile, format_profile),
+            "key": CacheKey(context, content_profile, format_profile, err_profile),
+            "element": ElementName(context, content_profile, format_profile),
+            "action": TypeName(context, content_profile, format_profile),
+            "message": MessageOrLogFile(context, content_profile, format_profile, err_profile)
         }
-        logfile_tokens = self._parse_logfile_format(message_format, content_profile, format_profile)
+        logfile_tokens = self._parse_logfile_format(context.log_message_format, content_profile, format_profile)
         self._columns.extend(logfile_tokens)
 
     # show_pipeline()
@@ -390,8 +370,6 @@ class LogLine(Widget):
             else:
                 if element._cached():
                     line = p.fmt_subst(line, 'state', "cached", fg='magenta')
-                elif element._remotely_cached():
-                    line = p.fmt_subst(line, 'state', "downloadable", fg='cyan')
                 elif consistency == Consistency.RESOLVED:
                     line = p.fmt_subst(line, 'state', "fetch needed", fg='red')
                 elif element._buildable():
@@ -454,23 +432,17 @@ class LogLine(Widget):
     # and so on.
     #
     # Args:
-    #    pipeline (Pipeline): The pipeline to print the heading of
+    #    project (Project): The toplevel project we were invoked from
+    #    stream (Stream): The stream
     #    log_file (file): An optional file handle for additional logging
-    #    deps (list): Optional list of elements, default is to use the whole pipeline
     #    styling (bool): Whether to enable ansi escape codes in the output
     #
-    def print_heading(self, pipeline, log_file, deps=None, styling=False):
-        context = pipeline.context
-        project = pipeline.project
+    def print_heading(self, project, stream, *, log_file, styling=False):
+        context = self.context
         starttime = datetime.datetime.now()
         text = ''
 
-        assert self._resolved_keys is None
-        elements = set()
-        visited = {}
-        for element in pipeline.targets:
-            elements.update(element.dependencies(Scope.ALL, visited=visited))
-        self._resolved_keys = {element: element._get_cache_key() for element in elements}
+        self._resolved_keys = {element: element._get_cache_key() for element in stream.session_elements}
 
         # Main invocation context
         text += '\n'
@@ -478,7 +450,7 @@ class LogLine(Widget):
         values = OrderedDict()
         values["Session Start"] = starttime.strftime('%A, %d-%m-%Y at %H:%M:%S')
         values["Project"] = "{} ({})".format(project.name, project.directory)
-        values["Targets"] = ", ".join([t.name for t in pipeline.targets])
+        values["Targets"] = ", ".join([t.name for t in stream.targets])
         text += self._format_values(values)
 
         # User configurations
@@ -513,9 +485,7 @@ class LogLine(Widget):
 
         # Pipeline state
         text += self.content_profile.fmt("Pipeline\n", bold=True)
-        if deps is None:
-            deps = pipeline.dependencies(Scope.ALL)
-        text += self.show_pipeline(deps, context.log_element_format)
+        text += self.show_pipeline(stream.total_elements, context.log_element_format)
         text += '\n'
 
         # Separator line before following output
@@ -530,17 +500,16 @@ class LogLine(Widget):
     # Print a summary of activities at the end of a session
     #
     # Args:
-    #    pipeline (Pipeline): The Pipeline
-    #    scheduler (Scheduler): The Scheduler
+    #    stream (Stream): The Stream
     #    log_file (file): An optional file handle for additional logging
     #    styling (bool): Whether to enable ansi escape codes in the output
     #
-    def print_summary(self, pipeline, scheduler, log_file, styling=False):
+    def print_summary(self, stream, log_file, styling=False):
 
         # Early silent return if there are no queues, can happen
-        # only in the case that the pipeline early returned due to
+        # only in the case that the stream early returned due to
         # an inconsistent pipeline state.
-        if scheduler.queues is None:
+        if not stream.queues:
             return
 
         text = ''
@@ -549,7 +518,7 @@ class LogLine(Widget):
         elements = sorted(e for (e, k) in self._resolved_keys.items() if k != e._get_cache_key())
         if elements:
             text += self.content_profile.fmt("Resolved key Summary\n", bold=True)
-            text += self.show_pipeline(elements, self._context.log_element_format)
+            text += self.show_pipeline(elements, self.context.log_element_format)
             text += "\n\n"
 
         if self._failure_messages:
@@ -563,18 +532,18 @@ class LogLine(Widget):
         text += self.content_profile.fmt("Pipeline Summary\n", bold=True)
         values = OrderedDict()
 
-        values['Total'] = self.content_profile.fmt(str(pipeline.total_elements))
-        values['Session'] = self.content_profile.fmt(str(pipeline.session_elements))
+        values['Total'] = self.content_profile.fmt(str(len(stream.total_elements)))
+        values['Session'] = self.content_profile.fmt(str(len(stream.session_elements)))
 
         processed_maxlen = 1
         skipped_maxlen = 1
         failed_maxlen = 1
-        for queue in scheduler.queues:
+        for queue in stream.queues:
             processed_maxlen = max(len(str(len(queue.processed_elements))), processed_maxlen)
             skipped_maxlen = max(len(str(len(queue.skipped_elements))), skipped_maxlen)
             failed_maxlen = max(len(str(len(queue.failed_elements))), failed_maxlen)
 
-        for queue in scheduler.queues:
+        for queue in stream.queues:
             processed = str(len(queue.processed_elements))
             skipped = str(len(queue.skipped_elements))
             failed = str(len(queue.failed_elements))
@@ -604,14 +573,6 @@ class LogLine(Widget):
     ###################################################
     #             Widget Abstract Methods             #
     ###################################################
-    def size_request(self, pipeline):
-        for widget in self._columns:
-            widget.size_request(pipeline)
-
-        self._space_widget.size_request(pipeline)
-        self._logfile_widget.size_request(pipeline)
-
-        self._context = pipeline.context
 
     def render(self, message):
 
@@ -630,7 +591,7 @@ class LogLine(Widget):
         logfile_tokens = []
         while format_string:
             if format_string.startswith("%%"):
-                logfile_tokens.append(FixedText("%", content_profile, format_profile))
+                logfile_tokens.append(FixedText(self.context, "%", content_profile, format_profile))
                 format_string = format_string[2:]
                 continue
             m = re.search(r"^%\{([^\}]+)\}", format_string)
@@ -643,7 +604,7 @@ class LogLine(Widget):
             else:
                 m = re.search("^[^%]+", format_string)
                 if m is not None:
-                    text = FixedText(m.group(0), content_profile, format_profile)
+                    text = FixedText(self.context, m.group(0), content_profile, format_profile)
                     format_string = format_string[m.end(0):]
                     logfile_tokens.append(text)
                 else:
@@ -710,7 +671,7 @@ class LogLine(Widget):
         if message.scheduler and message.message_type == MessageType.FAIL:
             text += '\n'
 
-            if self._context is not None and not self._context.log_verbose:
+            if self.context is not None and not self.context.log_verbose:
                 text += self._indent + self._err_profile.fmt("Log file: ")
                 text += self._indent + self._logfile_widget.render(message) + '\n'
             else:
