@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2016-2018 Codethink Limited
 #
@@ -29,17 +28,19 @@ from . import _yaml
 from ._profile import Topics, profile_start, profile_end
 from ._exceptions import LoadError, LoadErrorReason
 from ._options import OptionPool
-from ._artifactcache import artifact_cache_specs_from_config_node
+from ._artifactcache import ArtifactCache
 from ._elementfactory import ElementFactory
 from ._sourcefactory import SourceFactory
 from ._projectrefs import ProjectRefs, ProjectRefStorage
 from ._versions import BST_FORMAT_VERSION
-from ._workspaces import Workspaces
 
 
 # The separator we use for user specified aliases
 _ALIAS_SEPARATOR = ':'
 
+# Project Configuration file
+_PROJECT_CONF_FILE = 'project.conf'
+
 
 # HostMount()
 #
@@ -75,7 +76,7 @@ class Project():
         self.name = None
 
         # The project directory
-        self.directory = os.path.abspath(directory)
+        self.directory = self._ensure_project_dir(directory)
 
         # Absolute path to where elements are loaded from within the project
         self.element_path = None
@@ -84,7 +85,6 @@ class Project():
         self.refs = ProjectRefs(self.directory, 'project.refs')
         self.junction_refs = ProjectRefs(self.directory, 'junction.refs')
 
-        self.workspaces = None                   # Workspaces
         self.options = None                      # OptionPool
         self.junction = junction                 # The junction Element object, if this is a subproject
         self.fail_on_overlap = False             # Whether overlaps are treated as errors
@@ -211,7 +211,7 @@ class Project():
     def _load(self):
 
         # Load builtin default
-        projectfile = os.path.join(self.directory, "project.conf")
+        projectfile = os.path.join(self.directory, _PROJECT_CONF_FILE)
         config = _yaml.load(_site.default_project_config)
 
         # Load project local config and override the builtin
@@ -296,10 +296,7 @@ class Project():
         #
 
         # Load artifacts pull/push configuration for this project
-        self.artifact_cache_specs = artifact_cache_specs_from_config_node(config)
-
-        # Workspace configurations
-        self.workspaces = Workspaces(self)
+        self.artifact_cache_specs = ArtifactCache.specs_from_config_node(config, self.directory)
 
         # Plugin origins and versions
         origins = _yaml.node_get(config, list, 'plugins', default_value=[])
@@ -459,10 +456,26 @@ class Project():
                 origin_dict['path'] = os.path.join(self.directory, origin_dict['path'])
             destination.append(origin_dict)
 
-    def _extract_plugin_paths(self, node, name):
-        if not node:
-            return
-        path_list = _yaml.node_get(node, list, name, default_value=[])
-        for i in range(len(path_list)):
-            path = _yaml.node_get(node, str, name, indices=[i])
-            yield path
+    # _ensure_project_dir()
+    #
+    # Returns path of the project directory, if a configuration file is found
+    # in given directory or any of its parent directories.
+    #
+    # Args:
+    #    directory (str) - directory from where the command was invoked
+    #
+    # Raises:
+    #    LoadError if project.conf is not found
+    #
+    def _ensure_project_dir(self, directory):
+        directory = os.path.abspath(directory)
+        while not os.path.isfile(os.path.join(directory, _PROJECT_CONF_FILE)):
+            parent_dir = os.path.dirname(directory)
+            if directory == parent_dir:
+                raise LoadError(
+                    LoadErrorReason.MISSING_PROJECT_CONF,
+                    '{} not found in current directory or any of its parent directories'
+                    .format(_PROJECT_CONF_FILE))
+            directory = parent_dir
+
+        return directory
