@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2016-2018 Codethink Limited
 #
@@ -19,8 +18,8 @@
 #        Tristan Van Berkom <tristan.vanberkom@codethink.co.uk>
 
 """
-Element
-=======
+Element - Base element class
+============================
 
 
 .. _core_element_abstract_methods:
@@ -66,7 +65,7 @@ Miscellaneous abstract methods also exist:
 
 * :func:`Element.generate_script() <buildstream.element.Element.generate_script>`
 
-  For the purpose of ``bst source bundle``, an Element may optionally implmenent this.
+  For the purpose of ``bst source bundle``, an Element may optionally implement this.
 
 
 Class Reference
@@ -215,18 +214,17 @@ class Element(Plugin):
         self.__consistency = Consistency.INCONSISTENT  # Cached overall consistency state
         self.__cached = None                    # Whether we have a cached artifact
         self.__strong_cached = None             # Whether we have a cached artifact
-        self.__remotely_cached = None           # Whether we have a remotely cached artifact
-        self.__remotely_strong_cached = None    # Whether we have a remotely cached artifact
         self.__assemble_scheduled = False       # Element is scheduled to be assembled
         self.__assemble_done = False            # Element is assembled
         self.__tracking_scheduled = False       # Sources are scheduled to be tracked
         self.__tracking_done = False            # Sources have been tracked
-        self.__pull_failed = False              # Whether pull was attempted but failed
-        self.__log_path = None                  # Path to dedicated log file or None
+        self.__pull_done = False                # Whether pull was attempted
         self.__splits = None                    # Resolved regex objects for computing split domains
         self.__whitelist_regex = None           # Resolved regex object to check if file is allowed to overlap
         self.__staged_sources_directory = None  # Location where Element.stage_sources() was called
         self.__tainted = None                   # Whether the artifact is tainted and should not be shared
+        self.__required = False                 # Whether the artifact is required in the current session
+        self.__artifact_size = None             # The size of data committed to the artifact cache
 
         # hash tables of loaded artifact metadata, hashed by key
         self.__metadata_keys = {}                     # Strong and weak keys for this key
@@ -477,7 +475,11 @@ class Element(Plugin):
           name = self.node_subst_member(node, 'name')
         """
         value = self.node_get_member(node, str, member_name, default)
-        return self.__variables.subst(value)
+        try:
+            return self.__variables.subst(value)
+        except LoadError as e:
+            provenance = _yaml.node_get_provenance(node, key=member_name)
+            raise LoadError(e.reason, '{}: {}'.format(provenance, str(e))) from e
 
     def node_subst_list(self, node, member_name):
         """Fetch a list from a node member, substituting any variables in the list
@@ -497,7 +499,14 @@ class Element(Plugin):
         perform variable substitutions.
         """
         value = self.node_get_member(node, list, member_name)
-        return [self.__variables.subst(x) for x in value]
+        ret = []
+        for index, x in enumerate(value):
+            try:
+                ret.append(self.__variables.subst(x))
+            except LoadError as e:
+                provenance = _yaml.node_get_provenance(node, key=member_name, indices=[index])
+                raise LoadError(e.reason, '{}: {}'.format(provenance, str(e))) from e
+        return ret
 
     def node_subst_list_element(self, node, member_name, indices):
         """Fetch the value of a list element from a node member, substituting any variables
@@ -534,7 +543,11 @@ class Element(Plugin):
                   node, 'strings', [ i ])
         """
         value = self.node_get_list_element(node, str, member_name, indices)
-        return self.__variables.subst(value)
+        try:
+            return self.__variables.subst(value)
+        except LoadError as e:
+            provenance = _yaml.node_get_provenance(node, key=member_name, indices=indices)
+            raise LoadError(e.reason, '{}: {}'.format(provenance, str(e))) from e
 
     def compute_manifest(self, *, include=None, exclude=None, orphans=True):
         """Compute and return this element's selective manifest
@@ -659,7 +672,6 @@ class Element(Plugin):
         overlaps = OrderedDict()
         files_written = {}
         old_dep_keys = {}
-        project = self._get_project()
         workspace = self._get_workspace()
 
         if self.__can_build_incrementally() and workspace.last_successful:
@@ -688,7 +700,7 @@ class Element(Plugin):
                     # In case we are running `bst shell`, this happens in the
                     # main process and we need to update the workspace config
                     if utils._is_main_process():
-                        project.workspaces.save_config()
+                        self._get_context().get_workspaces().save_config()
 
             result = dep.stage_artifact(sandbox,
                                         path=path,
@@ -939,15 +951,6 @@ class Element(Plugin):
     def _cached(self):
         return self.__cached
 
-    # _remotely_cached():
-    #
-    # Returns:
-    #    (bool): Whether this element is already present in
-    #            the remote artifact cache
-    #
-    def _remotely_cached(self):
-        return self.__remotely_cached
-
     # _buildable():
     #
     # Returns:
@@ -958,7 +961,12 @@ class Element(Plugin):
             return False
 
         for dependency in self.dependencies(Scope.BUILD):
-            if not dependency._cached():
+            # In non-strict mode an element's strong cache key may not be available yet
+            # even though an artifact is available in the local cache. This can happen
+            # if the pull job is still pending as the remote cache may have an artifact
+            # that matches the strict cache key, which is preferred over a locally
+            # cached artifact with a weak cache key match.
+            if not dependency._cached() or not dependency._get_cache_key(strength=_KeyStrength.STRONG):
                 return False
 
         if not self.__assemble_scheduled:
@@ -1029,8 +1037,6 @@ class Element(Plugin):
             self.__weak_cache_key = None
             self.__strict_cache_key = None
             self.__strong_cached = None
-            self.__remotely_cached = None
-            self.__remotely_strong_cached = None
             return
 
         if self.__weak_cache_key is None:
@@ -1061,9 +1067,8 @@ class Element(Plugin):
             # are sufficient. However, don't update the `cached` attributes
             # until the full cache query below.
             cached = self.__artifacts.contains(self, self.__weak_cache_key)
-            remotely_cached = self.__artifacts.remote_contains(self, self.__weak_cache_key)
             if (not self.__assemble_scheduled and not self.__assemble_done and
-                    not cached and not remotely_cached):
+                    not cached and not self._pull_pending() and self._is_required()):
                 self._schedule_assemble()
                 return
 
@@ -1081,15 +1086,11 @@ class Element(Plugin):
         key_for_cache_lookup = self.__strict_cache_key if context.get_strict() else self.__weak_cache_key
         if not self.__cached:
             self.__cached = self.__artifacts.contains(self, key_for_cache_lookup)
-        if not self.__remotely_cached:
-            self.__remotely_cached = self.__artifacts.remote_contains(self, key_for_cache_lookup)
         if not self.__strong_cached:
             self.__strong_cached = self.__artifacts.contains(self, self.__strict_cache_key)
-        if not self.__remotely_strong_cached:
-            self.__remotely_strong_cached = self.__artifacts.remote_contains(self, self.__strict_cache_key)
 
         if (not self.__assemble_scheduled and not self.__assemble_done and
-                not self.__cached and not self.__remotely_cached):
+                not self.__cached and not self._pull_pending() and self._is_required()):
             # Workspaced sources are considered unstable if a build is pending
             # as the build will modify the contents of the workspace.
             # Determine as early as possible if a build is pending to discard
@@ -1154,7 +1155,7 @@ class Element(Plugin):
     def _preflight(self):
 
         if self.BST_FORBID_RDEPENDS and self.BST_FORBID_BDEPENDS:
-            if any(self.dependencies(Scope.RUN, recurse=False)) or any(self.dependencies(Scope.RUN, recurse=False)):
+            if any(self.dependencies(Scope.RUN, recurse=False)) or any(self.dependencies(Scope.BUILD, recurse=False)):
                 raise ElementError("{}: Dependencies are forbidden for '{}' elements"
                                    .format(self, self.get_kind()), reason="element-forbidden-depends")
 
@@ -1241,8 +1242,7 @@ class Element(Plugin):
     # is used to stage things by the `bst checkout` codepath
     #
     @contextmanager
-    def _prepare_sandbox(self, scope, directory, integrate=True):
-
+    def _prepare_sandbox(self, scope, directory, deps='run', integrate=True):
         with self.__sandbox(directory, config=self.__sandbox_config) as sandbox:
 
             # Configure always comes first, and we need it.
@@ -1254,15 +1254,16 @@ class Element(Plugin):
                     self.stage(sandbox)
                 elif scope == Scope.RUN:
                     # Stage deps in the sandbox root
-                    with self.timed_activity("Staging dependencies", silent_nested=True):
-                        self.stage_dependency_artifacts(sandbox, scope)
+                    if deps == 'run':
+                        with self.timed_activity("Staging dependencies", silent_nested=True):
+                            self.stage_dependency_artifacts(sandbox, scope)
 
-                    # Run any integration commands provided by the dependencies
-                    # once they are all staged and ready
-                    if integrate:
-                        with self.timed_activity("Integrating sandbox"):
-                            for dep in self.dependencies(scope):
-                                dep.integrate(sandbox)
+                        # Run any integration commands provided by the dependencies
+                        # once they are all staged and ready
+                        if integrate:
+                            with self.timed_activity("Integrating sandbox"):
+                                for dep in self.dependencies(scope):
+                                    dep.integrate(sandbox)
 
             yield sandbox
 
@@ -1320,15 +1321,45 @@ class Element(Plugin):
         # Ensure deterministic owners of sources at build time
         utils._set_deterministic_user(directory)
 
+    # _set_required():
+    #
+    # Mark this element and its runtime dependencies as required.
+    # This unblocks pull/fetch/build.
+    #
+    def _set_required(self):
+        if self.__required:
+            # Already done
+            return
+
+        self.__required = True
+
+        # Request artifacts of runtime dependencies
+        for dep in self.dependencies(Scope.RUN, recurse=False):
+            dep._set_required()
+
+        self._update_state()
+
+    # _is_required():
+    #
+    # Returns whether this element has been marked as required.
+    #
+    def _is_required(self):
+        return self.__required
+
     # _schedule_assemble():
     #
     # This is called in the main process before the element is assembled
     # in a subprocess.
     #
     def _schedule_assemble(self):
+        assert self._is_required()
         assert not self.__assemble_scheduled
         self.__assemble_scheduled = True
 
+        # Requests artifacts of build dependencies
+        for dep in self.dependencies(Scope.BUILD, recurse=False):
+            dep._set_required()
+
         # Invalidate workspace key as the build modifies the workspace directory
         workspace = self._get_workspace()
         if workspace:
@@ -1360,12 +1391,21 @@ class Element(Plugin):
             # For this reason, it is safe to update and
             # save the workspaces configuration
             #
-            project = self._get_project()
             key = self._get_cache_key()
             workspace = self._get_workspace()
             workspace.last_successful = key
             workspace.clear_running_files()
-            project.workspaces.save_config()
+            self._get_context().get_workspaces().save_config()
+
+            # We also need to update the required artifacts, since
+            # workspaced dependencies do not have a fixed cache key
+            # when the build starts.
+            #
+            # This does *not* cause a race condition, because
+            # _assemble_done is called before a cleanup job may be
+            # launched.
+            #
+            self.__artifacts.append_required_artifacts([self])
 
     # _assemble():
     #
@@ -1460,8 +1500,9 @@ class Element(Plugin):
                 utils.link_files(collectdir, filesdir)
 
                 # Copy build log
-                if self.__log_path:
-                    shutil.copyfile(self.__log_path, os.path.join(logsdir, 'build.log'))
+                log_filename = context.get_log_filename()
+                if log_filename:
+                    shutil.copyfile(log_filename, os.path.join(logsdir, 'build.log'))
 
                 # Store public data
                 _yaml.dump(_yaml.node_sanitize(self.__dynamic_public), os.path.join(metadir, 'public.yaml'))
@@ -1494,6 +1535,7 @@ class Element(Plugin):
                 }), os.path.join(metadir, 'workspaced-dependencies.yaml'))
 
                 with self.timed_activity("Caching artifact"):
+                    self.__artifact_size = utils._get_dir_size(assembledir)
                     self.__artifacts.commit(self, assembledir, self.__get_cache_keys_for_commit())
 
             # Finally cleanup the build dir
@@ -1507,31 +1549,55 @@ class Element(Plugin):
     #   (bool): Whether a pull operation is pending
     #
     def _pull_pending(self):
-        if self.__pull_failed:
-            # Consider this equivalent to artifact being unavailable in
-            # remote cache
+        if self.__strong_cached:
+            # Artifact already in local cache
             return False
 
-        if not self.__strong_cached and self.__remotely_strong_cached:
-            # Pull pending using strict cache key
-            return True
-        elif not self.__cached and self.__remotely_cached:
-            # Pull pending using weak cache key
-            return True
-        else:
-            # No pull pending
-            return False
+        # Pull is pending if artifact remote server available
+        # and pull has not been attempted yet
+        return self.__artifacts.has_fetch_remotes(element=self) and not self.__pull_done
 
-    # _pull_failed()
+    # _pull_done()
     #
-    # Indicate that pull was attempted but failed.
+    # Indicate that pull was attempted.
     #
-    # This needs to be called in the main process after
-    # a pull fails so that we properly update the main
+    # This needs to be called in the main process after a pull
+    # succeeds or fails so that we properly update the main
     # process data model
     #
-    def _pull_failed(self):
-        self.__pull_failed = True
+    # This will result in updating the element state.
+    #
+    def _pull_done(self):
+        self.__pull_done = True
+
+        self._update_state()
+
+    def _pull_strong(self, *, progress=None):
+        weak_key = self._get_cache_key(strength=_KeyStrength.WEAK)
+
+        key = self.__strict_cache_key
+        if not self.__artifacts.pull(self, key, progress=progress):
+            return False
+
+        # update weak ref by pointing it to this newly fetched artifact
+        self.__artifacts.link_key(self, key, weak_key)
+
+        return True
+
+    def _pull_weak(self, *, progress=None):
+        weak_key = self._get_cache_key(strength=_KeyStrength.WEAK)
+
+        if not self.__artifacts.pull(self, weak_key, progress=progress):
+            return False
+
+        # extract strong cache key from this newly fetched artifact
+        self._pull_done()
+
+        # create tag for strong cache key
+        key = self._get_cache_key(strength=_KeyStrength.STRONG)
+        self.__artifacts.link_key(self, weak_key, key)
+
+        return True
 
     # _pull():
     #
@@ -1545,26 +1611,14 @@ class Element(Plugin):
         def progress(percent, message):
             self.status(message)
 
-        weak_key = self._get_cache_key(strength=_KeyStrength.WEAK)
+        # Attempt to pull artifact without knowing whether it's available
+        pulled = self._pull_strong(progress=progress)
 
-        if self.__remotely_strong_cached:
-            key = self.__strict_cache_key
-            self.__artifacts.pull(self, key, progress=progress)
+        if not pulled and not self._cached() and not context.get_strict():
+            pulled = self._pull_weak(progress=progress)
 
-            # update weak ref by pointing it to this newly fetched artifact
-            self.__artifacts.link_key(self, key, weak_key)
-        elif not context.get_strict() and self.__remotely_cached:
-            self.__artifacts.pull(self, weak_key, progress=progress)
-
-            # extract strong cache key from this newly fetched artifact
-            self._update_state()
-
-            # create tag for strong cache key
-            key = self._get_cache_key(strength=_KeyStrength.STRONG)
-            self.__artifacts.link_key(self, weak_key, key)
-        else:
-            raise ElementError("Attempt to pull unavailable artifact for element {}"
-                               .format(self.name))
+        if not pulled:
+            return False
 
         # Notify successfull download
         display_key = self.__get_brief_display_key()
@@ -1590,16 +1644,7 @@ class Element(Plugin):
         if self.__get_tainted():
             return True
 
-        # Use the strong cache key to check whether a remote already has the artifact.
-        # In non-strict mode we want to push updated artifacts even if the
-        # remote already has an artifact with the same weak cache key.
-        key = self._get_cache_key(strength=_KeyStrength.STRONG)
-
-        # Skip if every push remote contains this element already.
-        if self.__artifacts.push_needed(self, key):
-            return False
-        else:
-            return True
+        return False
 
     # _push():
     #
@@ -1618,7 +1663,14 @@ class Element(Plugin):
 
         with self.timed_activity("Pushing artifact"):
             # Push all keys used for local commit
-            return self.__artifacts.push(self, self.__get_cache_keys_for_commit())
+            pushed = self.__artifacts.push(self, self.__get_cache_keys_for_commit())
+            if not pushed:
+                return False
+
+            # Notify successful upload
+            display_key = self.__get_brief_display_key()
+            self.info("Pushed artifact {}".format(display_key))
+            return True
 
     # _shell():
     #
@@ -1720,8 +1772,27 @@ class Element(Plugin):
     #    (Workspace|None): A workspace associated with this element
     #
     def _get_workspace(self):
-        project = self._get_project()
-        return project.workspaces.get_workspace(self.name)
+        workspaces = self._get_context().get_workspaces()
+        return workspaces.get_workspace(self._get_full_name())
+
+    # _get_artifact_size()
+    #
+    # Get the size of the artifact produced by this element in the
+    # current pipeline - if this element has not been assembled or
+    # pulled, this will be None.
+    #
+    # Note that this is the size of an artifact *before* committing it
+    # to the cache, the size on disk may differ. It can act as an
+    # approximate guide for when to do a proper size calculation.
+    #
+    # Returns:
+    #    (int|None): The size of the artifact
+    #
+    def _get_artifact_size(self):
+        return self.__artifact_size
+
+    def _get_artifact_cache(self):
+        return self.__artifacts
 
     # _write_script():
     #
@@ -1766,46 +1837,17 @@ class Element(Plugin):
     def _subst_string(self, value):
         return self.__variables.subst(value)
 
-    # Run some element methods with logging directed to
-    # a dedicated log file, here we yield the filename
-    # we decided on for logging
+    # Returns the element whose sources this element is ultimately derived from.
     #
-    @contextmanager
-    def _logging_enabled(self, action_name):
-        self.__log_path = self.__logfile(action_name)
-        with open(self.__log_path, 'a') as logfile:
-
-            # Write one last line to the log and flush it to disk
-            def flush_log():
-
-                # If the process currently had something happening in the I/O stack
-                # then trying to reenter the I/O stack will fire a runtime error.
-                #
-                # So just try to flush as well as we can at SIGTERM time
-                try:
-                    logfile.write('\n\nAction {} for element {} forcefully terminated\n'
-                                  .format(action_name, self.name))
-                    logfile.flush()
-                except RuntimeError:
-                    os.fsync(logfile.fileno())
-
-            self._set_log_handle(logfile)
-            with _signals.terminator(flush_log):
-                yield self.__log_path
-            self._set_log_handle(None)
-            self.__log_path = None
-
-    # Override plugin _set_log_handle(), set it for our sources and dependencies too
+    # This is intended for being used to redirect commands that operate on an
+    # element to the element whose sources it is ultimately derived from.
     #
-    # A log handle is set once in the context of a child task which will have only
-    # one log, so it's not harmful to modify the state of dependencies
-    def _set_log_handle(self, logfile, recurse=True):
-        super()._set_log_handle(logfile)
-        for source in self.sources():
-            source._set_log_handle(logfile)
-        if recurse:
-            for dep in self.dependencies(Scope.ALL):
-                dep._set_log_handle(logfile, False)
+    # For example, element A is a build element depending on source foo,
+    # element B is a filter element that depends on element A. The source
+    # element of B is A, since B depends on A, and A has sources.
+    #
+    def _get_source_element(self):
+        return self
 
     #############################################################
     #                   Private Local Methods                   #
@@ -1821,17 +1863,11 @@ class Element(Plugin):
         if self.__tracking_scheduled:
             return
 
-        # Determine overall consistency of the element
-        consistency = Consistency.CACHED
-        for source in self.__sources:
-            source._update_state()
-            source_consistency = source._get_consistency()
-            consistency = min(consistency, source_consistency)
-        self.__consistency = consistency
+        self.__consistency = Consistency.CACHED
+        workspace = self._get_workspace()
 
         # Special case for workspaces
-        workspace = self._get_workspace()
-        if workspace and self.__consistency > Consistency.INCONSISTENT:
+        if workspace:
 
             # A workspace is considered inconsistent in the case
             # that it's directory went missing
@@ -1839,6 +1875,13 @@ class Element(Plugin):
             fullpath = workspace.get_absolute_path()
             if not os.path.exists(fullpath):
                 self.__consistency = Consistency.INCONSISTENT
+        else:
+
+            # Determine overall consistency of the element
+            for source in self.__sources:
+                source._update_state()
+                source_consistency = source._get_consistency()
+                self.__consistency = min(self.__consistency, source_consistency)
 
     # __calculate_cache_key():
     #
@@ -1876,11 +1919,15 @@ class Element(Plugin):
                 'execution-environment': self.__sandbox_config.get_unique_key(),
                 'environment': cache_env,
                 'sources': [s._get_unique_key(workspace is None) for s in self.__sources],
-                'workspace': '' if workspace is None else workspace.get_key(),
+                'workspace': '' if workspace is None else workspace.get_key(self._get_project()),
                 'public': self.__public,
                 'cache': type(self.__artifacts).__name__
             }
 
+            # fail-on-overlap setting cannot affect elements without dependencies
+            if project.fail_on_overlap and dependencies:
+                self.__cache_key_dict['fail-on-overlap'] = True
+
         cache_key_dict = self.__cache_key_dict.copy()
         cache_key_dict['dependencies'] = dependencies
 
@@ -1895,7 +1942,7 @@ class Element(Plugin):
     #    (bool): Whether this element can be built incrementally
     #
     def __can_build_incrementally(self):
-        return self._get_workspace() and self.__artifacts.can_diff()
+        return bool(self._get_workspace())
 
     # __get_brief_display_key():
     #
@@ -1927,43 +1974,6 @@ class Element(Plugin):
             if workspace:
                 workspace.prepared = True
 
-    # __logfile()
-    #
-    # Compose the log file for this action & pid.
-    #
-    # Args:
-    #    action_name (str): The action name
-    #    pid (int): Optional pid, current pid is assumed if not provided.
-    #
-    # Returns:
-    #    (string): The log file full path
-    #
-    # Log file format, when there is a cache key, is:
-    #
-    #    '{logdir}/{project}/{element}/{cachekey}-{action}.{pid}.log'
-    #
-    # Otherwise, it is:
-    #
-    #    '{logdir}/{project}/{element}/{:0<64}-{action}.{pid}.log'
-    #
-    # This matches the order in which things are stored in the artifact cache
-    #
-    def __logfile(self, action_name, pid=None):
-        project = self._get_project()
-        context = self._get_context()
-        key = self.__get_brief_display_key()
-        if pid is None:
-            pid = os.getpid()
-
-        action = action_name.lower()
-        logfile = "{key}-{action}.{pid}.log".format(
-            key=key, action=action, pid=pid)
-
-        directory = os.path.join(context.logdir, project.name, self.normal_name)
-
-        os.makedirs(directory, exist_ok=True)
-        return os.path.join(directory, logfile)
-
     # __assert_cached()
     #
     # Raises an error if the artifact is not cached.
