@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #  Copyright (C) 2017 Codethink Limited
 #
 #  This program is free software; you can redistribute it and/or
@@ -17,7 +16,13 @@
 #  Authors:
 #        Jonathan Maw <jonathan.maw@codethink.co.uk>
 
-"""A source implementation for staging bazaar branches
+"""
+bzr - stage files from a bazaar repository
+==========================================
+
+**Host dependencies:**
+
+  * bzr
 
 **Usage:**
 
