@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2016 Codethink Limited
 #
@@ -18,7 +17,13 @@
 #  Authors:
 #        Tristan Van Berkom <tristan.vanberkom@codethink.co.uk>
 
-"""A Source implementation for staging git checkouts
+"""
+git - stage files from a git repository
+=======================================
+
+**Host dependencies:**
+
+  * git
 
 **Usage:**
 
