@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 #
 #  Copyright (C) 2016 Codethink Limited
 #
@@ -18,8 +17,8 @@
 #  Authors:
 #        Tristan Van Berkom <tristan.vanberkom@codethink.co.uk>
 """
-Source
-======
+Source - Base source class
+==========================
 
 
 .. _core_source_abstract_methods:
