@@ -9,6 +9,7 @@ MAIN_COMMANDS = [
     'build ',
     'checkout ',
     'fetch ',
+    'help ',
     'init ',
     'pull ',
     'push ',
@@ -174,8 +175,9 @@ def test_option_directory(datafiles, cli, cmd, word_idx, expected, subdir):
      ['compose-all.bst ', 'compose-include-bin.bst ', 'compose-exclude-dev.bst '], None),
 
     # When running from the files subdir
-    ('project', 'bst show ', 2, [], 'files'),
-    ('project', 'bst build com', 2, [], 'files'),
+    ('project', 'bst show ', 2, [e + ' ' for e in PROJECT_ELEMENTS], 'files'),
+    ('project', 'bst build com', 2,
+     ['compose-all.bst ', 'compose-include-bin.bst ', 'compose-exclude-dev.bst '], 'files'),
 
     # When passing the project directory
     ('project', 'bst --directory ../ show ', 4, [e + ' ' for e in PROJECT_ELEMENTS], 'files'),
@@ -193,8 +195,10 @@ def test_option_directory(datafiles, cli, cmd, word_idx, expected, subdir):
      ['compose-all.bst ', 'compose-include-bin.bst ', 'compose-exclude-dev.bst '], None),
 
     # When running from the files subdir
-    ('no-element-path', 'bst show ', 2, [], 'files'),
-    ('no-element-path', 'bst build com', 2, [], 'files'),
+    ('no-element-path', 'bst show ', 2,
+     [e + ' ' for e in (PROJECT_ELEMENTS + ['project.conf'])] + ['files/'], 'files'),
+    ('no-element-path', 'bst build com', 2,
+     ['compose-all.bst ', 'compose-include-bin.bst ', 'compose-exclude-dev.bst '], 'files'),
 
     # When passing the project directory
     ('no-element-path', 'bst --directory ../ show ', 4,
@@ -213,3 +217,16 @@ def test_argument_element(datafiles, cli, project, cmd, word_idx, expected, subd
     if subdir:
         cwd = os.path.join(cwd, subdir)
     assert_completion(cli, cmd, word_idx, expected, cwd=cwd)
+
+
+@pytest.mark.parametrize("cmd,word_idx,expected", [
+    ('bst he', 1, ['help ']),
+    ('bst help ', 2, MAIN_COMMANDS),
+    ('bst help fe', 2, ['fetch ']),
+    ('bst help p', 2, ['pull ', 'push ']),
+    ('bst help p', 2, ['pull ', 'push ']),
+    ('bst help w', 2, ['workspace ']),
+    ('bst help workspace ', 3, WORKSPACE_COMMANDS),
+])
+def test_help_commands(cli, cmd, word_idx, expected):
+    assert_completion(cli, cmd, word_idx, expected)
