@@ -59,6 +59,42 @@ def test_build_checkout(datafiles, cli, strict, hardlinks):
     assert os.path.exists(filename)
 
 
+@pytest.mark.datafiles(DATA_DIR)
+@pytest.mark.parametrize("deps", [("run"), ("none")])
+def test_build_checkout_deps(datafiles, cli, deps):
+    project = os.path.join(datafiles.dirname, datafiles.basename)
+    checkout = os.path.join(cli.directory, 'checkout')
+
+    # First build it
+    result = cli.run(project=project, args=['build', 'target.bst'])
+    result.assert_success()
+
+    # Assert that after a successful build, the builddir is empty
+    builddir = os.path.join(cli.directory, 'build')
+    assert os.path.isdir(builddir)
+    assert not os.listdir(builddir)
+
+    # Now check it out
+    result = cli.run(project=project, args=['checkout', 'target.bst', '--deps', deps, checkout])
+    result.assert_success()
+
+    # Check that the executable hello file is found in the checkout
+    filename = os.path.join(checkout, 'usr', 'bin', 'hello')
+
+    if deps == "run":
+        assert os.path.exists(filename)
+    else:
+        assert not os.path.exists(filename)
+
+    # Check that the executable hello file is found in the checkout
+    filename = os.path.join(checkout, 'usr', 'include', 'pony.h')
+
+    if deps == "run":
+        assert os.path.exists(filename)
+    else:
+        assert not os.path.exists(filename)
+
+
 @pytest.mark.datafiles(DATA_DIR)
 @pytest.mark.parametrize("hardlinks", [("copies"), ("hardlinks")])
 def test_build_checkout_nonempty(datafiles, cli, hardlinks):
@@ -88,7 +124,7 @@ def test_build_checkout_nonempty(datafiles, cli, hardlinks):
 
     # Now check it out
     result = cli.run(project=project, args=checkout_args)
-    result.assert_main_error(ErrorDomain.PIPELINE, None)
+    result.assert_main_error(ErrorDomain.STREAM, None)
 
 
 @pytest.mark.datafiles(DATA_DIR)
@@ -192,7 +228,7 @@ def test_install_to_build(cli, tmpdir, datafiles):
     # attempt to stage into /buildstream/build, which is not allowed.
     result = cli.run(project=project, args=strict_args(['build', element], True))
 
-    result.assert_main_error(ErrorDomain.PIPELINE, None)
+    result.assert_main_error(ErrorDomain.STREAM, None)
     result.assert_task_error(ErrorDomain.ELEMENT, None)
 
 
@@ -390,3 +426,22 @@ def test_build_checkout_workspaced_junction(cli, tmpdir, datafiles):
     with open(filename, 'r') as f:
         contents = f.read()
     assert contents == 'animal=Horsy\n'
+
+
+@pytest.mark.datafiles(DATA_DIR)
+def test_build_checkout_cross_junction(datafiles, cli, tmpdir):
+    project = os.path.join(datafiles.dirname, datafiles.basename)
+    subproject_path = os.path.join(project, 'files', 'sub-project')
+    junction_path = os.path.join(project, 'elements', 'junction.bst')
+    checkout = os.path.join(cli.directory, 'checkout')
+
+    generate_junction(tmpdir, subproject_path, junction_path)
+
+    result = cli.run(project=project, args=['build', 'junction.bst:import-etc.bst'])
+    result.assert_success()
+
+    result = cli.run(project=project, args=['checkout', 'junction.bst:import-etc.bst', checkout])
+    result.assert_success()
+
+    filename = os.path.join(checkout, 'etc', 'animal.conf')
+    assert os.path.exists(filename)
