@@ -1,5 +1,7 @@
-import subprocess
+import os
 import pytest
+import shutil
+import subprocess
 
 from .repo import Repo
 from ..site import HAVE_GIT
@@ -36,6 +38,14 @@ class Git(Repo):
                         env=GIT_ENV, cwd=self.repo)
         return self.latest_commit()
 
+    def add_file(self, filename):
+        shutil.copy(filename, self.repo)
+        subprocess.call(['git', 'add', os.path.basename(filename)], env=GIT_ENV, cwd=self.repo)
+        subprocess.call([
+            'git', 'commit', '-m', 'Added {}'.format(os.path.basename(filename))
+        ], env=GIT_ENV, cwd=self.repo)
+        return self.latest_commit()
+
     def add_submodule(self, subdir, url=None, checkout=None):
         submodule = {}
         if checkout is not None:
@@ -68,3 +78,6 @@ class Git(Repo):
             'git', 'rev-parse', 'master'
         ], env=GIT_ENV, cwd=self.repo)
         return output.decode('UTF-8').strip()
+
+    def branch(self, branch_name):
+        subprocess.call(['git', 'checkout', '-b', branch_name], env=GIT_ENV, cwd=self.repo)
